/*
 * Copyright 2013 eXo Platform SAS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package juzu;

import javax.inject.Singleton;
import java.lang.annotation.Annotation;

/** @author <a href="mailto:julien.viet@exoplatform.com">Julien Viet</a> */
public enum Scope {

  SINGLETON(Singleton.class) {
    @Override
    public boolean isBuiltIn() {
      return true;
    }
  },

  REQUEST(RequestScoped.class) {
    @Override
    public boolean isBuiltIn() {
      return false;
    }
  },

  SESSION(SessionScoped.class) {
    @Override
    public boolean isBuiltIn() {
      return false;
    }
  },

  /**
   * todo : study more in depth how flash scoped is propagated to other phase, specially the resource phase todo : that
   * should kind of have an ID.
   */
  FLASH(FlashScoped.class) {
    @Override
    public boolean isBuiltIn() {
      return false;
    }
  },

  IDENTITY(IdentityScoped.class) {
    @Override
    public boolean isBuiltIn() {
      return false;
    }
  };

  /** . */
  private final Class<? extends Annotation> annotationType;

  Scope(Class<? extends Annotation> annotationType) {
    this.annotationType = annotationType;
  }

  public abstract boolean isBuiltIn();

  public Class<? extends Annotation> getAnnotationType() {
    return annotationType;
  }
}
