/*
 * Copyright 2013 eXo Platform SAS
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package juzu.impl.template.spi;

import juzu.impl.common.Path;

import java.io.Serializable;
import java.util.LinkedHashSet;

/**
 * The representation of a compilable template in a context.
 *
 * @author <a href="mailto:julien.viet@exoplatform.com">Julien Viet</a>
 * @param <M> the template model
 */
public class Template<M extends Serializable> implements Serializable {

  /** The origin path. */
  private final Path.Relative origin;

  /** . */
  private final M model;

  /** . */
  private final Path.Relative relativePath;

  /** . */
  private final Path.Absolute absolutePath;

  /** . */
  private final LinkedHashSet<String> parameters;

  /** The last modified date. */
  private long lastModified;

  public Template(
    Path.Relative origin,
    M model,
    Path.Relative relativePath,
    Path.Absolute absolutePath,
    long lastModified) {
    this.origin = origin;
    this.model = model;
    this.relativePath = relativePath;
    this.parameters = new LinkedHashSet<String>();
    this.lastModified = lastModified;
    this.absolutePath = absolutePath;
  }

  public Path.Relative getOrigin() {
    return origin;
  }

  public Path.Absolute getAbsolutePath() {
    return absolutePath;
  }

  public Path.Relative getRelativePath() {
    return relativePath;
  }

  public M getModel() {
    return model;
  }

  public long getLastModified() {
    return lastModified;
  }

  public LinkedHashSet<String> getParameters() {
    return parameters;
  }

  public void addParameter(String parameterName) {
    parameters.add(parameterName);
  }
}
